
import os

import cartopy.crs as ccrs
import numpy as np
import skimage
import ucat
import xarray as xr

CASES = [
    ['DWD', 'Belchatow'],
    ['DWD', 'Jaenschwalde'],
    ['Empa', 'Belchatow'],
    ['Empa', 'Berlin'],
    ['Empa', 'Jaenschwalde'],
    ['Empa', 'Lipetsk'],
    ['Empa', 'Matimba'],
    ['Empa', 'Paris'],
    ['LSCE', 'Paris'],
    ['TNO', 'Belchatow'],
    ['TNO', 'Belchatow', 'wNO'],
    ['TNO', 'Berlin'],
    ['TNO', 'Jaenschwalde'],
    ['TNO', 'Jaenschwalde', 'wNO'],
    ['TNO', 'Randstad_S'],
    ['TNO', 'Randstad_W'],
    ['WUR', 'Belchatow'],
    ['WUR', 'Belchatow', 'wNO'],
    ['WUR', 'Jaenschwalde'],
    ['WUR', 'Jaenschwalde', 'wNO'],
    ['WUR', 'Lipetsk'],
    ['WUR', 'Lipetsk', 'wNO'],
    ['WUR', 'Matimba'],
    ['WUR', 'Matimba', 'wNO']
]

DATA_PATH = '/project/coco2/jupyter/WP4/powerplants/library_of_plumes/'


def read_level2(team, region, suffix='', data_path=DATA_PATH,
                co2_noise=0.7, no2_noise=2e15):
    """\
    Read CO2M-like Level-2 from CoCO2 library of plumes.

    DWD_Belchatow.nc            LSCE_Paris.nc            WUR_Belchatow_wNO.nc
    DWD_Jaenschwalde.nc         TNO_Belchatow.nc         WUR_Jaenschwalde.nc
    Empa_Belchatow.nc           TNO_Belchatow_wNO.nc     WUR_Jaenschwalde_wNO.nc
    Empa_Berlin.nc              TNO_Berlin.nc            WUR_Lipetsk.nc
    Empa_Jaenschwalde.nc        TNO_Jaenschwalde.nc      WUR_Lipetsk_wNO.nc
    Empa_Lipetsk.nc             TNO_Jaenschwalde_wNO.nc  WUR_Matimba.nc
    Empa_Matimba.nc             TNO_Randstad_S.nc        WUR_Matimba_wNO.nc
    Empa_Paris.nc               TNO_Randstad_W.nc        WUR_Belchatow.nc
    """
    if suffix:
        filename = f'{team}_{region}_{suffix}.nc'
    else:
        filename = f'{team}_{region}.nc'

    filename = os.path.join(data_path, filename)

    # noise levels
    no2_noise = ucat.convert_columns(no2_noise, 'cm-2', 'mol m-2',
                                     molar_mass='NO2')

    d = xr.open_dataset(filename)

    d = d.rename_vars({'lon_bnds': 'lonc', 'lat_bnds': 'latc',
                       'surface_pressure': 'psurf'})

    name = 'CITY' 
    if region in ['Berlin', 'Paris']:
        name = 'CITY'
    elif region in ['Randstad_S', 'Randstad_W']:
        name = 'RS'
    else:
        name = 'PP_M'

    shape = d[f'XCO2_{name}'].shape
    d['CO2'] = d[f'XCO2_{name}'] \
               + d.get('XCO2_ANTH', 0.0) \
               + d.get('XCO2_BIO', 0.0) \
               + d.get('XCO2_BG', 0.0) \
               + co2_noise * np.random.randn(*shape)

    d['CO2_signal'] = d[f'XCO2_{name}'].copy()
    d['CO2_std'] = xr.full_like(d['CO2'], co2_noise)
    d['CO2'].attrs['units'] = 'ppm'

    if f'NO2_{name}' in d and f'NO_{name}' in d:
        d['NOx'] = d[f'NO_{name}'] + d[f'NO2_{name}'] \
                   + d['NO_BG'] + d['NO2_BG'] \
                   + d.get('NO_ANTH', 0.0) + d.get('NO2_ANTH', 0.0) \
                   + d.get('NO_BIO', 0.0) + d.get('NO2_BIO', 0.0) \
                   + no2_noise * np.random.randn(*shape)
        d['NOx_signal'] = d[f'NO_{name}'] + d[f'NO2_{name}']
        d['NOx'].attrs['units'] = 'mol m-2'
        d['NOx_std'] = xr.full_like(d['NOx'], no2_noise)

    if f'NO2_{name}' in d:
        d['NO2'] = d[f'NO2_{name}'] \
                   + d['NO2_BG'] \
                   + d.get('NO2_ANTH', 0.0) \
                   + d.get('NO2_BIO', 0.0) \
                   + no2_noise * np.random.randn(*shape)
        d['NO2_signal'] = d[f'NO2_{name}'].copy()
        d['NO2'].attrs['units'] = 'mol m-2'
        d['NO2_std'] = xr.full_like(d['NO2'], no2_noise)

    d['clouds'] = xr.zeros_like(d['CO2'])

    # remove non-continous fields on boundary (only MicroHH)
    if team == 'WUR':
        mask = np.any(np.isnan(d['CO2'].values), axis=0)
        mask = skimage.morphology.dilation(mask, skimage.morphology.square(10))

        for name in ["CO2", "NOx", "NO2"]:
            if name in d:
                d[name].values[:,mask] = np.nan
                d[name + "_signal"].values[:,mask] = np.nan
                d[name + "_std"].values[:,mask] = np.nan

    return d






















